"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GraphBase = void 0;
const chart_js_1 = require("chart.js");
const types_1 = require("../types/types");
// type GraphType = "line" | "bar" | "doughnut";
/** An abstract that should be extends to generate Graph  */
class GraphBase {
    /**
     * @param {StatsManager} statsmanager
     * @param {GraphType} graphType - The GraphType
     * @param {GraphOptions} options - The options for the graph
     * @param {number} [options.height = 400] - The height of the graph
     * @param {number} [options.width = 800] - The width of the graph
     * @param {number} [options.time = 604800000] - The time in miliseconds that the graph will display default is 1 week
     * @param {string} [options.color = "rgba(227, 227, 227, 0.9)"] - The color of the text and borders
     * @param {string} [options.backgroundColor = "rgba(40, 40, 40, 0.9)"] - The color of the background
     * @param {number} [options.fontSize = 14] - The font size of the text
     */
    constructor(statsmanager, graphType, options = {}) {
        this.statsmanager = statsmanager;
        this.graphType = graphType;
        this.height = options.height || 400;
        this.width = options.width || 800;
        this.color = options.color || "rgba(227, 227, 227, 0.9)";
        this.time = options.time || 7 * 24 * 60 * 60 * 1000;
        this.backgroundColor = options.backgroundColor || "rgba(40, 40, 40, 0.9)";
        chart_js_1.Chart.defaults.color = options.color || "rgba(227, 227, 227, 0.9)";
        chart_js_1.Chart.defaults.font.size = options.fontSize || 14;
    }
    /**
     * @param {string} targetStats - The name of the stats to display
     * @returns {Promise<Buffer>} - The graph buffer
     */
    async generate(targetStats) {
        const data = await this.statsmanager.getStats();
        if (data.length === 0)
            throw new Error("No data");
        const module = this.statsmanager.findModule(targetStats);
        if (!module)
            throw new Error(`Module ${targetStats} not found or not enabled`);
        if (this.graphType === types_1.GraphType.LINE && module.dataType === "number") {
            return this.generateCanvas(data.map(d => {
                if (!module.validInputData(d.stats[module.name]))
                    throw new Error(`Invalid data type for module ${targetStats}`);
                return { timestamp: d.timestamp, value: d.stats[module.name] };
            }));
        }
        else if (this.graphType === types_1.GraphType.BAR && module.dataType === "map") {
            return this.generateCanvas(data.map(d => {
                const mapData = new Map(Object.entries(d.stats[module.name]));
                if (!module.validInputData(mapData))
                    throw new Error(`Invalid data type for module ${targetStats}`);
                return { timestamp: d.timestamp, value: mapData };
            }));
        }
        else if (this.graphType === types_1.GraphType.DOUGHNUT && module.dataType === "map") {
            return this.generateCanvas(data.map(d => {
                const mapData = new Map(Object.entries(d.stats[module.name]));
                if (!module.validInputData(mapData))
                    throw new Error(`Invalid data type for module ${targetStats}`);
                return { timestamp: d.timestamp, value: mapData };
            }));
        }
        throw new Error(`Invalid graph type for module ${targetStats}`);
    }
}
exports.GraphBase = GraphBase;
//# sourceMappingURL=BaseGraph.js.map